#
# This file is part of PackageManager-Virtual
#
# This software is copyright (c) 2019 by Daniel Maurice Davis.
#
# This is free software; you can redistribute it and/or modify it under
# the same terms as the Perl 5 programming language system itself.
#
# This Build.PL for PackageManager-Virtual was generated by Dist::Zilla::Plugin::ModuleBuildTiny 0.015.
use strict;
use warnings;

use 5.006;
use Module::Build::Tiny 0.034;
Build_PL();
